const byte ClockStep = 4; //Stepper motor driver step pin
const byte ClockEna = 2;  //Stepper motor driver enable pin
const byte ClockDir = 3;  //Stepper motor driver direction pin
const byte ClockUp = 0;   //Time up switch
const byte ClockDown = 1; //Time down switch

const long ClockStepsPerHour = 3200; //the amount of steps an hour takes
const long ClockEnaDelay = 10000; //the time in microseconds the stepper will stay enabled after a step
const long ClockFastDelay = 1000; //the time in microseconds between steps in fast mode.
const int ClockSwitchModeTime = 3000; //the time in milliseconds the up and down need to be held to switch modes

byte ClockMode = 0; //the clock mode states the normal mode (0), or fast mode (1).
unsigned long ClockNormalTarget;
unsigned long ClockNormalDelay;


void setup() {
  pinMode(ClockStep, OUTPUT);
  pinMode(ClockEna, OUTPUT);
  pinMode(ClockDir, OUTPUT);
  pinMode(ClockUp, INPUT);
  pinMode(ClockDown, INPUT);

  digitalWrite(ClockDir, 0);

  //calculate normal step delay
  ClockNormalDelay = 3600000 / ClockStepsPerHour;
}

void loop() {
  //if the sysyem is in normal mode
  if (ClockMode == 0) {
    if (millis() > ClockNormalTarget) { //if target is reached
      digitalWrite(ClockEna, 0); //enable motor
      digitalWrite(ClockStep, 1); //step the motor
      digitalWrite(ClockStep, 0);

      //set new target
      ClockNormalTarget += ClockNormalDelay;

      delayMicroseconds(ClockEnaDelay); //wait for the step to be completed
      digitalWrite(ClockEna, 1); //disable the motor
    }
  }

  //if the system is in fast mode
  if (ClockMode == 1) {
    digitalWrite(ClockEna, 0); //enable motor

    //step the motor
    digitalWrite(ClockStep, 1); //step the motor
    digitalWrite(ClockStep, 0);

    //delay for the configured time
    delayMicroseconds(ClockFastDelay);
  }

  //Move the time up
  if (digitalRead(ClockUp) == 1 && digitalRead(ClockDown) == 0) {
    digitalWrite(ClockDir, 0);
    digitalWrite(ClockEna, 0); //enable motor
    //step the motor
    digitalWrite(ClockStep, 1); //step the motor
    digitalWrite(ClockStep, 0);
    //delay for the configured time
    delayMicroseconds(ClockFastDelay);
  }

  //move the time down
  if (digitalRead(ClockUp) == 0 && digitalRead(ClockDown) == 1) {
    digitalWrite(ClockDir, 1);
    digitalWrite(ClockEna, 0); //enable motor
    //step the motor
    digitalWrite(ClockStep, 1); //step the motor
    digitalWrite(ClockStep, 0);
    //delay for the configured time
    delayMicroseconds(ClockFastDelay);
  }

  //switch modes
  if (digitalRead(ClockUp) == 1 && digitalRead(ClockDown) == 1) {
    unsigned long ClockToggleTarget = millis() + ClockSwitchModeTime;
    while (digitalRead(ClockUp) == 1 & digitalRead(ClockDown) == 1) { //while loop, if it reaches the threshold, toggle modes
      if (millis() > ClockToggleTarget) {
        if (ClockMode == 0) {
          ClockMode = 1;
        }
        else {
          ClockMode = 0;
        }
        break;
      }
    }
  }
}

